clc
clear
open_system('Flapping_Sys')
%%%%%%%%%%%%%%%       Possible variables    %%%%%%%%%%%%%%%%%%%%%%

% Radius, Length, Stiffness (K_w), J0 & c_f ratio

%%%%%%%%%%%%%%%%%%%    Parameter setting initialisation    %%%%%%%%%%%%%

        set_param('Flapping_Sys/Slider-Crank & Wing/Fcn','Expression','Radius*sin(u(1))+sqrt(Length^2-Radius^2*(cos(u(1)))^2)')
        set_param('Flapping_Sys/Slider-Crank & Wing/Constant','value','Length')
        set_param('Flapping_Sys/Slider-Crank & Wing/Constant2','value','Length')

        set_param('Flapping_Sys/Slider-Crank & Wing/Fcn','Expression','Radius*sin(u(1))+sqrt(Length^2-Radius^2*(cos(u(1)))^2)')
        set_param('Flapping_Sys/Slider-Crank & Wing/Gain2','Gain','Radius')
        set_param('Flapping_Sys/Slider-Crank & Wing/Gain3','Gain','Radius')
        set_param('Flapping_Sys/Slider-Crank & Wing/Load mass','Gain','Radius')
        set_param('Flapping_Sys/Slider-Crank & Wing/Constant3','value','Radius/tan(Ang_beta*2*pi/360)')
        set_param('Flapping_Sys/Slider-Crank & Wing/Constant8','value','Radius/tan(Ang_beta*2*pi/360)')
        set_param('Flapping_Sys/Slider-Crank & Wing/Constant1','value','Radius/tan(Ang_beta*2*pi/360)')

        set_param('Flapping_Sys/Slider-Crank & Wing/K_w','Gain' , 'K_w')


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%           Input      %%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%   Activate parameters

% xfreq = 0; %If x-axis is parameters
 xfreq = 1; % if x-axis is the frequency

 Parameter = 'Leng';        % Length
% Parameter = 'Radi';       % Raidus
% Parameter = 'Rati';       % Ratio 
% Parameter = 'Stif';        % Stiffness  - input frequencies to calculate
                             % the stiffness (K_w)

T_s = 1; % simulation stop time
Simul_step = 10^(-6); % Fixed step in simulation setting

Vmin = 1.5;     % 1.2, 3.4;     % Min input voltage (Vs)
Vmax = 3.5;     % 1;       % Max input voltage
Vstep = 1;  % 0.4;       % Iteration step

Pmin =  20e-3;%10e-3; % Parameter min
Pmax =  20e-3;
Pstep = 10e-3; % Parameter step

%%%%%%%%%%%%%%%%%%% Initial values of control variables %%%%%%%%%%%%%%%%%

Radius = 5e-3; % Radius of connector[m]
Length = 20e-3; % Length of Bar connecting motor and wing[m]

rho = 1.225; % air density

J0 = 7.205e-8;
D_wing =0.5*rho*5.35e-8;  % For drag torque (Wing 10 to 60)
L_wing = 0.5*rho*1.3045e-6;  % For lift %integral from 10e-3 to 60e-3, (r^2)*(31e-3)*sqrt(1-(r^2)/((60e-3)^2)) 
% aerodynamic centre is obtained using lift 

Spring = 0; %22

ur = 2.4e-3; %2.5e-3; % friction coefficient

Ang_beta = 40; % wing stroke angle [deg]

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%           System        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

set_param('Flapping_Sys' , 'StopTime', 'T_s') 
time_period_double = T_s/Simul_step+1;
time_period = int64(time_period_double);


%%%%%%%%             Parameters Initial setting          %%%%%%%%%%%%

Ra = 9;      % Terminal Resistance

L = 29e-6; % Terminal inductance  
ng = 25;    % gear ratio 25

Kt = 1.0e-2; %1.29e-2;% Nm/A Torque constant 7.664 mNm/A(paper))
Ke = Kt; 

J = 2.4e-10; 1.77e-10;  % Rotor inertia kgm 
Jg = 1e-10;    % Gear inertia kgm

bm =  3.6e-6;  % mechanical loss on motor and gear 5.08e-6;

H = Radius/tan(Ang_beta*2*pi/360);         % The distance between hinge and slider) mm

K_w = (J0)*(Spring*2*pi)^2; % stiffness %
%%%%%%%%%%%%%%%%%%%%% variable sizing %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

No_step_double = (Vmax-Vmin)/ Vstep +1; % the number of steps for voltage iteration
No_step = int64(No_step_double);

No_P_step_double = (Pmax-Pmin)/ Pstep +1; % the number of steps for parameter iteration
No_P_step = int64(No_P_step_double);

%%%%%%%%% variables (array) 

D_Pe(time_period,No_step,No_P_step) = 0;
D_Pm(time_period,No_step,No_P_step) = 0;
D_P_inertial(time_period,No_step,No_P_step) = 0;
D_P_aero(time_period,No_step,No_P_step) = 0;
D_Lift(time_period,No_step,No_P_step) = 0;
D_Drag(time_period,No_step,No_P_step) = 0;

D_Speed(time_period,No_step,No_P_step) = 0;
D_Torque(time_period,No_step,No_P_step) = 0;
D_KmIW(time_period,No_step,No_P_step) = 0;
D_P_loss_cal(time_period,No_step,No_P_step) = 0;
D_T_L(time_period,No_step,No_P_step) = 0;


%%%%%%%%%%%%%%%%%%%%%%   Iteration   %%%%%%%%%%%%
%%%%%%%%%%%%%%% data transfer to match the matrix size
i = 1;

for Vs = Vmin : Vstep : Vmax
    j = 1;
    
    for Para = Pmin : Pstep : Pmax 
        
        % Parameter setting in the simulink model 
% Check after the simulation whether the parameter value is 1
        Check_Length = 0;
        Check_Radius = 0;
        Check_Ratio = 0;
        Check_Stiffness = 0;

        set_param('Flapping_Sys/Vs' , 'value', 'Vs') 

        if Parameter == 'Leng' 
        set_param('Flapping_Sys/Slider-Crank & Wing/Fcn','Expression','Radius*sin(u(1))+sqrt(Para^2-Radius^2*(cos(u(1)))^2)')
        set_param('Flapping_Sys/Slider-Crank & Wing/Constant','value','Para')
        set_param('Flapping_Sys/Slider-Crank & Wing/Constant2','value','Para')

        Check_Length = 1;
        
        elseif Parameter == 'Radi'
        set_param('Flapping_Sys/Slider-Crank & Wing/Fcn','Expression','Para*sin(u(1))+sqrt(Length^2-Para^2*(cos(u(1)))^2)')
        set_param('Flapping_Sys/Slider-Crank & Wing/Gain2','Gain','Para')
        set_param('Flapping_Sys/Slider-Crank & Wing/Gain3','Gain','Para')
        set_param('Flapping_Sys/Slider-Crank & Wing/Load mass','Gain','Para')
        set_param('Flapping_Sys/Slider-Crank & Wing/Constant3','value','Para/tan(Ang_beta*2*pi/360)')
        set_param('Flapping_Sys/Slider-Crank & Wing/Constant8','value','Para/tan(Ang_beta*2*pi/360)')
        set_param('Flapping_Sys/Slider-Crank & Wing/Constant1','value','Para/tan(Ang_beta*2*pi/360)')

        Check_Radius = 1;
        
        elseif Parameter == 'Rati'
        set_param('Flapping_Sys/Slider-Crank & Wing/J0','Gain','J0 * Para')
        set_param('Flapping_Sys/Slider-Crank & Wing/c_f','Gain' , 'c_f * Para')

        Check_Ratioo = 1;
        
        elseif Parameter == 'Stif'
            K_w = J0*(Para*2*pi)^2;
        set_param('Flapping_Sys/Slider-Crank & Wing/K_w','Gain' , 'K_w')

        Check_Stiffness = 1;
        end
        
        sim('Flapping_Sys.slx')    
    
        D_pe_broker = Pe.data;
        D_Pe(:,i,j)= D_pe_broker;
        
        D_KmIW_broker = KmIW.data;
        D_KmIW(:,i,j)= D_KmIW_broker;
        
        D_Lift_broker = Lift.data;
        D_Lift(:,i,j)= D_Lift_broker;
        
        D_Drag_broker = Drag.data;
        D_Drag(:,i,j)= D_Drag_broker;

        D_pm_broker = Pm.data;
        D_Pm(:,i,j)= D_pm_broker;

        D_P_inertial_broker = P_inertial.data;
        D_P_inertial(:,i,j) = D_P_inertial_broker;

        D_P_aero_broker = P_aero.data;
        D_P_aero(:,i,j) = D_P_aero_broker;

        D_Speed_broker = Speed.data;
        D_Speed(:,i,j) = D_Speed_broker;

        D_Torque_broker = Torque.data;
        D_Torque(:,i,j) = D_Torque_broker;
        
        D_T_L_broker = T_L.data;
        D_T_L(:,i,j) = D_T_L_broker;

        D_Displacement_broker = Displacement.data;
        D_Displacement(:,i,j) = D_Displacement_broker;
        
        D_P_loss_cal_broker=P_loss_cal.data;
        D_P_loss_cal(:,i,j) = D_P_loss_cal_broker;
        
        Pgraph(:,i,j) = Para;

        j=j+1;
    end
    
    i=i+1;
end


%%%%%%%%%%%%%%%% Data counting to get 1 period, based on the displacement
  for i = 1 : 1 : No_step
     for j = 1 : 1 : No_P_step        
         co = 0;
         No_p = 0;
         No = 0;

         for n = time_period-1 : -1 : 2
             No_p = No_p + 1;% The number of one period

             if  D_Displacement(n,i,j)> D_Displacement(n+1,i,j) && D_Displacement(n,i,j)> D_Displacement(n-1,i,j)
                co = co + 1;
             end
             if co == 0
                 No_p = 0;
             end
             if co == 2
                 No = No_p; % round(No_p/2,0); % for one period
                 break
             end
         end
         
                 T_D_Pe(time_period,No_step,No_P_step) =0;
                 T_D_KmIW(time_period,No_step,No_P_step) =0;
                 T_D_Lift(time_period,No_step,No_P_step) =0;
                 T_D_Drag(time_period,No_step,No_P_step) =0;
                 T_D_Pm(time_period,No_step,No_P_step) =0;
                 T_D_P_aero(time_period,No_step,No_P_step) =0;
                 T_D_P_inertial(time_period,No_step,No_P_step) =0;
                 T_D_Speed(time_period,No_step,No_P_step) =0;
                 T_D_Torque(time_period,No_step,No_P_step) =0;
                 T_D_Displacement(time_period,No_step,No_P_step) =0;
                 T_D_P_loss_cal(time_period,No_step,No_P_step) =0;
                 T_D_T_L(time_period,No_step,No_P_step) =0;
                 
                 No_pm(No_step,No_P_step) = 0;
                 TEST(time_period,No_step,No_P_step) =0;
                 
                 Mean_Pe(:,i,j) = 0;
         
         for t = time_period - No + 1 : 1 : time_period

            T_D_Pe(t-(time_period - No),i,j) = D_Pe(t,i,j);
            Mean_Pe(:,i,j) = mean(T_D_Pe(1:No,i,j));% electrical power
            
            T_D_Pm(t-(time_period - No),i,j) = D_Pm(t,i,j);            
            Mean_Pm(:,i,j) = mean(T_D_Pm(1:No,i,j));
            
            T_D_KmIW(t-(time_period - No),i,j) = D_KmIW(t,i,j);            
            Mean_KmIW(:,i,j) = mean(T_D_KmIW(1:No,i,j));
            
            T_D_T_L(t-(time_period - No),i,j) = D_T_L(t,i,j);            
            Mean_T_L(:,i,j) = mean(T_D_T_L(1:No,i,j));
            
            T_D_Lift(t-(time_period - No),i,j) = D_Lift(t,i,j);            
            Mean_Lift(:,i,j) = mean(T_D_Lift(1:No,i,j));
            
            T_D_Drag(t-(time_period - No),i,j) = D_Drag(t,i,j);            
            Mean_Drag(:,i,j) = mean(T_D_Drag(1:No,i,j));
            
            T_D_P_aero(t-(time_period - No),i,j) = D_P_aero(t,i,j);
            Mean_P_aero(:,i,j) = mean (T_D_P_aero(1:No,i,j));                   % aerodynamic power
            
            T_D_P_inertial(t-(time_period - No),i,j) = D_P_inertial(t,i,j);
            Mean_P_inertial(:,i,j) = mean (T_D_P_inertial(1:No,i,j));           % Inertial Power
            
            T_D_Speed(t-(time_period - No),i,j) = D_Speed(t,i,j);
            Mean_Speed(:,i,j) = mean (T_D_Speed(1:No,i,j));                     % Motor speed 
            
            T_D_Torque(t-(time_period - No),i,j) = D_Torque(t,i,j);
            Mean_Torque(:,i,j) = mean(T_D_Torque(1:No,i,j));
            
            Mean_Eff(:,i,j) = Mean_P_aero(:,i,j)/Mean_Pe(:,i,j)*100;%mean(T_D_Eff(1:No,i,j));
            
            Mean_Eff_M(:,i,j) = Mean_Pm(:,i,j)/Mean_Pe(:,i,j)*100; % motor efficiency
            
            Mean_Eff_KmIW(:,i,j) = Mean_KmIW(:,i,j)/Mean_Pe(:,i,j)*100;
            
            Mean_Eff_Mech(:,i,j) = Mean_P_aero(:,i,j)/Mean_KmIW(:,i,j)*100;
            
            Mean_Ploss(:,i,j) = Mean_Pe(:,i,j) - Mean_P_aero(:,i,j);
            
            T_D_P_loss_cal(t-(time_period - No),i,j) = D_P_loss_cal(t,i,j);
            Mean_P_loss_cal(:,i,j) = mean(T_D_P_loss_cal(1:No,i,j)); 
       
            T_D_Displacement(t-(time_period - No),i,j) = D_Displacement(t,i,j);
            TEST(t-(time_period - No),i,j) = t - (time_period - No + 1) + 1; % to make a period graph

            No_pm(i,j) = No;

        end
     end
  end

if xfreq == 1 
    
Mean_Pe_gf(No_step) =0;
Mean_Pm_gf (No_step) =0;
Mean_P_aero_gf(No_step) =0;
Mean_P_inertial_gf(No_step) =0;
Mean_Eff_gf(No_step) =0;
Mean_Speed_gf(No_step) =0;
Mean_Lift_gf(No_step) =0;
Mean_Drag_gf(No_step) =0;

Mean_KmIW_gf(No_step) =0;
Mean_Eff_M_gf(No_step) =0;
Mean_Lift_per_VI_gf(No_step) =0;
Mean_Ploss_gf(No_step) =0;
Mean_P_loss_cal_gf(No_P_step) =0;
Mean_T_L_gf(No_P_step) =0;
Mean_Torque_gf(No_step) =0;
Mean_Eff_KmIW_gf(No_step) =0;
Mean_Eff_Mech_gf(No_step) =0;

for j = 1 : 1 : No_P_step
    for i = 1 : 1 : No_step
    
        No_g = No_pm(i,j);
        
        Mean_Pe_gf(i) = Mean_Pe(:,i,j);
        Mean_Pm_gf(i) = Mean_Pm(:,i,j);
        Mean_P_aero_gf(i) = Mean_P_aero(:,i,j);
        Mean_P_inertial_gf(i) = Mean_P_inertial(:,i,j);
        Mean_Eff_gf(i) = Mean_Eff(:,i,j);
        Mean_Speed_gf(i) = Mean_Speed(:,i,j);
        Mean_Lift_gf(i) = Mean_Lift(:,i,j);
        Mean_Drag_gf(i) = Mean_Drag(:,i,j);
        Mean_KmIW_gf(i) = Mean_KmIW(:,i,j);
        Mean_Eff_M_gf(i) = Mean_Eff_M(:,i,j);
        Mean_Ploss_gf(i) =  Mean_Ploss(:,i,j);
        Mean_P_loss_cal_gf(i) =  Mean_P_loss_cal(:,i,j);
        Mean_T_L_gf(i) = Mean_T_L(:,i,j);
        Mean_Torque_gf(i) = Mean_Torque(:,i,j);
            
        Mean_Eff_KmIW_gf(i) = Mean_Eff_KmIW(:,i,j);
        Mean_Eff_Mech_gf(i) = Mean_Eff_Mech(:,i,j);
        Mean_Lift_per_VI_gf(i) = Mean_Lift(:,i,j)/Mean_Pe(:,i,j);
        
    end

       %%%%%%%%%%%%%%%%%%%% Electrical power %%%%%%%%%%%
       plot (Mean_Speed_gf/2/pi, Mean_Pe_gf, 'square', 'DisplayName', sprintf('Electrical power, %.1fHz Spring, %.1f-%.1fV ' , Spring, Vmin, Vmax))
       hold on
       %%%%%%%%%%%%%%%%%%%% Lift %%%%%%%%%%%
       plot (Mean_Speed_gf/2/pi, Mean_Lift_gf*1000, '>', 'DisplayName', 'Lift')
       
%       plot(Mean_Speed_gf/2/pi,Mean_Lift*1000./Mean_Pe, 'DisplayName', sprintf('Lift per Pe %.2f %.2fHz', Vs, Spring))
%       plot(Mean_Speed_gf/2/pi,Mean_Lift*1000./Mean_Pm, 'DisplayName', sprintf('Lift per Pm %.2f %.2fHz', Vs, Spring))

        %%%%%%%%%%%%%%%%%%%% Drag %%%%%%%%%%%
%        plot (Mean_Speed_gf/2/pi, Mean_Drag_gf*1000, 'o', 'DisplayName', 'Drag')

        %%%%%%%%%%%%%%%%%%% Aerodynamic power %%%%%%%%%%%  
%       plot (Mean_Speed_gf/2/pi, Mean_P_aero_gf, '*', 'DisplayName', sprintf('Aerodynamic power, %.1fHz Spring', Spring))

        %%%%%%%%%%%%%%%%%%%% mechanical power %%%%%%%%%%% 
%       plot (Mean_Speed_gf/2/pi, Mean_Pm_gf, 'x', 'DisplayName', sprintf('Motor power, %.1fHz Spring', Spring))

%        %%%%%%%%%%%%%%%%%%%% Apparent inertial Power %%%%%%%%%%% 
%       plot (Mean_Speed_gf/2/pi, Mean_P_inertial_gf, 'o', 'DisplayName', sprintf( 'Absolute mean Reactive power, %.1fHz Spring', Spring))

%        %%%%%%%%%%%%%%%%%%%% Efficiency %%%%%%%%%%%
%       plot (Mean_Speed_gf/2/pi, Mean_Eff_gf, 'diamond', 'DisplayName', sprintf('Efficiency, %.1fHz Spring', Spring))

        %%%%%%%%%%%%%%%%%%%% KmIW %%%%%%%%%%%
        plot (Mean_Speed_gf/2/pi, Mean_KmIW_gf, '>', 'DisplayName', sprintf('KmIW, %.1fHz Spring', Spring))

         %%%%%%%%%%%%%%%%%%%% Motor efficiency %%%%%%%%%%%
%        plot (Mean_Speed_gf/2/pi, Mean_Eff_M_gf, 'hexagram', 'DisplayName',sprintf( 'Motor Efficiency, %.1fHz Spring', Spring))

%        %%%%%%%%%%%%%%%%%%%% Ploss %%%%%%%%%%%
%        plot (Mean_Speed_gf/2/pi, Mean_Ploss_gf, 'pentagram', 'DisplayName',sprintf( 'Pe-Paero, %.1fHz Spring', Spring))

%        %%%%%%%%%%%%%%%%%%%% Ploss calculation%%%%%%%%%%%
%        plot (Mean_Speed_gf/2/pi, Mean_P_loss_cal_gf, '<', 'DisplayName',sprintf( 'Pe-Pm, %.1fHz Spring', Spring))
%        
%        %%%%%%%%%%%%%%%%%%%% T_L %%%%%%%%%%%
%        plot (Mean_Speed_gf/2/pi, Mean_T_L_gf, 'pentagram', 'DisplayName',sprintf( 'T_L, %.1fHz Spring', Spring))
% 
%        %%%%%%%%%%%%%%%%%%%% Mean Torque %%%%%%%%%%%
%        plot (Mean_Speed_gf/2/pi, Mean_Torque_gf, 'pentagram', 'DisplayName',sprintf( 'Torque, %.1fHz Spring', Spring))
% 
%        plot (Mean_Speed_gf/2/pi, Mean_Eff_KmIW_gf, 'square', 'DisplayName', sprintf('Eff_KmIW, %.1fHz Spring, %.1f-%.1fV ' , Spring, Vmin, Vmax))
% 
%        plot (Mean_Speed_gf/2/pi, Mean_Eff_Mech_gf, 'square', 'DisplayName', sprintf('Eff_Mech, %.1fHz Spring, %.1f-%.1fV ' , Spring, Vmin, Vmax))


    end
end
